--
--BC_beta_1.e
--
--INTEGERS ONLY
--------------------------------------------------------------------
--                        Base Conversion                         --
--                             with                               --
--                          BC_beta_1.e                           --
--jxliv7@hotmail.com                                   2005-12-27 --
--------------------------------------------------------------------
------------
--Documentation
------------
--
--The sequences in Euphoria lend themselves to to consider
--      numbers by the digit, with each slice of a sequence
--      holding one digit. With the maximum value of an Euphoria
--      integer being 1,073,741,823, it's easy to consider that
--      number as a sequence: {1,0,7,3,7,4,1,8,2,3}
--
--E.g., 1234 would be {1,2,3,4}, but the decimal number 1234
--      becomes {1,0,0,1,1,0,1,0,0,1,0} base 2. That binary
--      number, 10011010010, is larger than the maximum value of
--      an integer. What would 1,073,741,823 be in binary?
--
--1234 base 10 is {2,3,2,2} base 8, {4,13,2} base 16, and
--      {1,4,16} base 87. Yes, 1234 base 10 is 4D2 in hexadecimal,
--      but it's easier to think of each slice in decimal (not to
--      mention easier to compute).
--
--Therefor, one limit is that each slice be less than or equal 
--      to the value of an integer, 1,073,741,823.
--
--Please also remember that for this include file, bases are always
--      positive integers between 2 and 1073741823, (although
--      there are undoubtedly some mathematician somewhere
--      who would disagree).
--
--This include file is called BC_beta_1.e. It has 3 purposes at
--      this point in time:
--          convert an integer N (where 2 <= N <= 1073741823 in
--              base 10) into a sequence
--          convert that sequence (where N is base 10) into a
--              sequence of digits in another base (where the
--              base is between 2 and 1073741823, inclusive.
--          convert a sequence (where N is in some base between
--              2 and 1073741823) into an integer sequence base 10.
--
--One note about the 2 main routines, Dec_Num() and Num_Dec():
--      if you're going to convert from AA base 5 to CC base 57,
--      it's much easier to convert AA base 5 to BB base 10, and
--      then convert BB base 10 to CC base 57. 
--
--Future versions of BC will be able to convert real numbers
--      (plus or minus numbers including decimal places) and
--      integers from one base to another. They should also be
--      more flexible with input (keyboard and disk) and output
--      (screen or disk).
--
------------
--Code
------------
--
with trace      --For your testing conveience & troubleshooting
--trace(1)
--
------------
------------
function revv(sequence s)
--
--This routine doesn't need the misc.e include file
--
integer lo
sequence t
    lo = 1
    t = repeat(0,length(s))
--
--trace(1)      --For your testing conveience
--trace(0)
for hi = length(s) to floor(length(s)/2)+1 by -1 do
    t[hi] = s[lo]
    t[lo] = s[hi]
    lo += 1
end for
--
return t
--
end function
--
------------
------------
global function Dec_Num(integer num,integer newbase)
--
--Convert:
--  an integer base 10 to an integer (2 <= base <= 1,073,741,822)
--
--Usage:  Sequence = DecToNum(num, newbase)
--
--E.g., convert 1234 base 10 to {1,4,16} base 87
--
--All numbers are sequences. 1234 >> {1,2,3,4}    
--
--This function uses the standard divide by the base and take
--      the remainder conversion method.
--
sequence new
atom n,rem
    new = {0}
    n = num
--
--trace(1)      --For your testing conveience
--trace(0)
for i = 1 to num by 1 do
    n = n/newbase
    rem = floor(((n*newbase)-(floor(n)*newbase))+.5)
--      In some cases, rem becomes a floating point number
--      which is why the floor(x,+.5) is there
    n = floor(n)
    new &= rem
    if n =0 then
        exit
    end if
end for
--
return revv(new[2..$])
--
end function
--
------------
------------
function TrimZeros(sequence s)
--
--There's no easy way to figure out how many digits a
--      number will be, so the leading zeros are cut
--
--trace(1)      --For your testing conveience
--trace(0)
for i = 1 to length(s) do
    while s[1] = 0 do
        s = s[2..$]
    end while
return s
end for
--
end function
--
------------
------------
global function Num_Dec(sequence num,integer oldbase)
--
--Convert:
--  an integer (2 <= base <= 1,073,741,822) to an integer base 10
--
--Usage:  Sequence = NumToDec(num, oldbase)
--
--E.g., convert 1122 base 11 to 1234 base 10
--All numbers are sequences. 1234 >> {1,2,3,4}    
--
--This function uses the standard multiply by the base and add
--      conversion method.
--
integer j
sequence new
    j = 0
    new ={0}
--
--trace(1)      --For your testing conveience
--trace(0)
for i = length(num) to 1 by -1 do
    new[1] += power(oldbase,j)*num[i]
    j += 1  
end for
--
new = repeat(0,10) & new[1]
--      There's no easy way to figure out how many digits a
--      number will be, hence 10+1 is chosen
for i=length(new) to 1 by -1 do
    while new[i] > 9 do
        new[i-1] += floor(new[i]/10)
        new[i] -= new[i-1]*10
    end while
end for
--
return TrimZeros(new)
--
end function
--
------------
------------
global function Num_Seq(integer num)
--
--Convert:
--  a number <= 1,073,741,822 to a sequence
--
--Usage:  Sequence = NumToSeq(num)
--
--E.g., convert 1,234,567 to {1,2,3,4,5,6,7}
--All numbers are sequences. 1234 >> {1,2,3,4}    
--
integer n, j
sequence new
    n = num
    j = 1
    new = {num}
--
--trace(1)      --For your testing conveience
--trace(0)
while n > 1 do
    new &= 0
    new[j+1] = floor(new[j]/10) 
    new[j] -= new[j+1]*10
    n = floor(n/10)
    j += 1
end while
new = revv(new)
--
return TrimZeros(new)
--
end function
--
------------
------------
--

